package consume

import (
	"os"
	"query-service/internal/usecases/databaseinfo"
	"query-service/internal/usecases/produce"

	"git.science.uu.nl/datastrophe/broker"
	"git.science.uu.nl/datastrophe/objectstore"
	"git.science.uu.nl/datastrophe/query-conversion"
	queryexecution "git.science.uu.nl/datastrophe/query-execution"
)

// Service wraps consumer methods
// broker is Alice broker created in brockerdriver driver
type Service struct {
	objectStore         objectstore.Interface
	brokerDriver        broker.Interface
	producer            produce.UseCase
	queryConverter      query.Converter
	requestSender       queryexecution.Executor
	databaseInfoService databaseinfo.UseCase
}

// NewService creates a new service
func NewService(objectStore objectstore.Interface, broker broker.Interface, produceService produce.UseCase, queryConverter query.Converter, requestSender queryexecution.Executor, databaseInfoService databaseinfo.UseCase) *Service {
	return &Service{
		objectStore:         objectStore,
		brokerDriver:        broker,
		producer:            produceService,
		queryConverter:      queryConverter,
		requestSender:       requestSender,
		databaseInfoService: databaseInfoService,
	}
}

// Start starts consuming
func (s *Service) Start() {
	// Create consumer
	var consumer broker.Consumer
	switch os.Getenv("QUERY_EXECUTION") {
	case "arangodb":
		consumer = s.brokerDriver.CreateConsumer("requests-exchange", "arangodb-query-queue", "arangodb-query-request")
	case "neo4j":
		consumer = s.brokerDriver.CreateConsumer("requests-exchange", "neo4j-query-queue", "neo4j-query-request")
	}

	consumer.SetMessageHandler(s.HandleMessage)

	// Start consuming messages
	consumer.ConsumeMessages()
}
